from tkinter import *
import multiprocessing as mp

class LEDMatrix:

    def __del__(self):
        self.continue_proc = False
        self.q.close()
    def __init__(self, rows = 8, cols = 8, color = (0,0,0), origin_x=5, origin_y=5,ledWidth =2, ledHeight=1, ledPadx=2,ledPady=2):

        self.origin_x = origin_x
        self.origin_y = origin_y
        self.__root = None
        self.continue_proc = True
        self.__rows = rows
        self.__cols = cols
        self.__ledWidth =ledWidth
        self.q = mp.Queue()
        self.__ledHeight = ledHeight
        self.__ledPadx = ledPadx
        self.__ledPady = ledPady
        self.__p = None
        self.__ledColor = self.__from_rgb(color)
        self.__matrix = [[0 for x in range(cols)] for y in range(rows)]
        self.__show()

    def __from_rgb(self, rgb):
        return "#%02x%02x%02x" % rgb

    def __show(self):

        self.__p = mp.Process(target=self._create_matrix, args=(self.q,))

        self.__p.start()

    def _create_matrix(self, q):
        row = 0
        col = 0
        self.__root = Tk()
        self.__root.overrideredirect(True)
        #self.__root.title = "LED Matrix"
        new_row = None

        def on_closing():
            self.continue_proc=False

        self.__root.protocol("WM_DELETE_WINDOW", on_closing)

        for color in range(self.__rows*self.__cols):

            e = Label(self.__root, text="", background=self.__ledColor,font=(None, -10))
            e.config(height=self.__ledHeight, width=self.__ledWidth)
            e.grid(row=row, column=col, padx=self.__ledPadx,pady=self.__ledPady)
            self.__matrix[row][col]=e
            #e.pack(expand=YES, fill=BOTH)
            col += 1
            if (col == self.__cols):
                row+=1
                col = 0

        self.__root.geometry('+%d+%d' % (self.origin_x, self.origin_y))
        while self.continue_proc:
            self.__root.update_idletasks()
            if q.qsize() > 0:
                action, *params = q.get(block=True)
                # Do Work
                if action == "set_pixel":
                    self.__matrix[params[1]][params[0]].config(background=self.__from_rgb(params[2]))
                elif action=="clear":
                    self.__clear()
            self.__root.update()

    def set_pixel(self, x, y, color):

        self.q.put(("set_pixel",x,y,color))

    def clear(self):

        self.q.put(("clear",(0,)))

    def __clear(self):
        for row in range(self.__rows):
            for col in range(self.__cols):
                self.__matrix[row][col].config(background=self.__ledColor)
                #self.__matrix[row][col].config(background=self.__ledColor)

if __name__ == "__main__":
    import time
    b = LEDMatrix(cols=16)

    while True:
        b.set_pixel(0,0,(255,0,0))
        b.set_pixel(0, 1, (0, 0, 0))
        time.sleep(5)

        b.set_pixel(0, 1, (255, 255, 0))
        time.sleep(2.7)
        b.set_pixel(0, 0, (0, 0, 0))
        b.set_pixel(0, 1, (0, 0, 0))
        b.set_pixel(0, 2, (0, 255, 0))
        time.sleep(5)
        b.set_pixel(0, 1, (255, 255, 0))
        b.set_pixel(0, 2, (0, 0, 0))
        time.sleep(2.7)


import time
from pihat_widget import image


class TrafficLight(image.ImageLoader):


    def set_state(self, state):
        """
        Set the state using a binary string
        :param state: binary string controlling red, amber green. 100 = (red(on) amber(off) green(off))
        :return: Exception on red, green and green amber
        """
        if state == "100":
            status = "red"
        elif state == "110":
            status ="redamber"
        elif state == "010":
            status = "amber"
        elif state == "001":
            status = "green"
        else:
            raise ValueError(f"Invalid Light State {state}")
        s = int(state, 2)
        self.q.put(status)

if __name__ == "__main__":

    lg = TrafficLight()

    while True:
        lg.set_state("100")

        time.sleep(5)
        lg.set_state("110")
        time.sleep(2.7)
        lg.set_state("001")
        time.sleep(5)
        lg.set_state("010")
        time.sleep(2.7)



from pihat_widget import image


class LEDCharacter(image.ImageLoader):

    def __init__(self,x,y):
        super().__init__(origin_x=x, origin_y=y, origin_width=45, origin_height=77, offsetx=0, offsety=0)
        self.__number = None

    @property
    def Number(self):
        return self.__number
    @Number.setter
    def Number(self, number):
        if type(number) is int:
            self.q.put(str(number))
            self.__number = number
        else:
            raise ValueError("Number must be an int")


class LEDMultiCharacter:

    def __init__(self, digits):

        self.__digits = digits
        self.__chars = [LEDCharacter(10+(i*45),10) for i in range(digits)]
        self.__number = None
        for z in self.__chars:
            z.Number =0

    @property
    def Number(self):
        return self.__number

    @Number.setter
    def Number(self, number):
        if type(number) is int:

            num = f"{number:0{self.__digits}d}"
            for c,n in enumerate(num):
                self.__chars[c].Number = int(n)

        else:
            raise ValueError("Number must be an int")
if __name__ == "__main__":


    c = LEDMultiCharacter(5)
    for i in range(65988):
        c.Number =i





from tkinter import *
import time
import multiprocessing as mp

class ImageLoader:

    def __del__(self):
        self.__continue_proc = False
        self.__q.close()


    @property
    def q(self):
        return self.__q

    def __init__(self,origin_x=5, origin_y=5, origin_width=140, origin_height=330, offsetx=10,offsety=10):

        self.__origin_x = origin_x
        self.__origin_y = origin_y
        self.__origin_width = origin_width
        self.__origin_height = origin_height
        self.__offset_x=offsetx
        self.__offset_y = offsety
        self.__continue_proc = True
        self.__q = mp.Queue()
        self.__p = None
        self.__root = None
        self.__canvas = None
        self.__show()

    def _display_image(self, q):
        self.__root = Tk()
        self.__root.geometry('+%d+%d' % (self.__origin_x, self.__origin_y))
        self.__root.overrideredirect(True)

        def __on_closing():
            self.__continue_proc=False


        self.__root.protocol("WM_DELETE_WINDOW", __on_closing)

        self.__canvas = Canvas(self.__root, width=self.__origin_width, height=self.__origin_height)
        self.__canvas.pack()

        while self.__continue_proc:
            self.__root.update_idletasks()
            if q.qsize()>0:
                result = q.get(block=True)

                img = PhotoImage(file=f".\\images\\{result}.png")

                self.__canvas.create_image(self.__offset_x, self.__offset_y, anchor=NW, image=img)
                #img.subsample(self.__zoom_x, self.__zoom_y)
            self.__root.update()


    def __show (self):

        self.__p=mp.Process(target=self._display_image, args=(self.__q,))

        self.__p.start()
        #p.join()





import pathlib
import setuptools

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setuptools.setup(
    name="pihat-widget",
    version="1.1.8",
    description="Widgets for Python programming practise",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/chrisf17/pihat-widgets",
    author="Chris Farrell",
    author_email="pihat@xivuh.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    packages=setuptools.find_packages(),


    include_package_data=True,
    install_requires=[],

)